// @flow
import React from 'react';
import autoBind from 'react-autobind';

import { connect } from 'react-redux';
import { compose, pure } from 'recompose';
import { changeEditorState as editorStateChange, toggleAlignment as alignmentToggle, linkPromptChangeState } from '../../actions/creators';

import { EditorState, DefaultDraftBlockRenderMap, RichUtils, ContentBlock } from 'draft-js';

import EditorArea from '../editorArea/editorArea';
import Toolbar from '../toolbar/toolbar';
import LinkPrompt from '../link/linkPrompt';

import styles from './editorComponent.module.scss';
import Separator from '../separator/separator';
import { handleReturnEmptyListItem, getActiveEntityAtCursor, removeCustomBlocksOnBackspace, reAlignBlock } from '../../utils/draftUtils';

export const blockRenderMap = DefaultDraftBlockRenderMap.merge({ hr: {}, blankAfterCustom: {} });


export function blockRendererFn(block: ContentBlock) {
  const type = block.getType();
  if (type === 'hr') {
    return {
      component: Separator,
      editable: false,
    };
  }
  return null;
}

export class EditorComponent extends React.Component {

  changeState: Function;

  constructor(){
    super();
    autoBind(this);
  }


  changeState(editorState: EditorState) {
    //TODO refactor to be shorter
    const { alignment, toggleAlignment, changeEditorState } = this.props;
    const newEditorState = reAlignBlock(editorState, alignment, toggleAlignment);
    const activeEntityAtCursor = getActiveEntityAtCursor(editorState);
    changeEditorState(newEditorState, null, activeEntityAtCursor);
  }

  onTab(e :Object){
    const { editorState, } = this.props;
    const newEditorState = RichUtils.onTab(e, editorState, 3);
    if (newEditorState !== editorState){
      this.changeState(newEditorState);
    }
  }

  handleKeyCommand(command :Object){
    const { editorState } = this.props;
    let newState = RichUtils.handleKeyCommand(editorState, command);
    newState = removeCustomBlocksOnBackspace(editorState, command, ['hr']);
    if (newState) {
      this.changeState(newState);
      return true;
    }
    return false;
  }


  handleReturn(e: Object){
    const { editorState } = this.props;
    return handleReturnEmptyListItem(editorState, this.changeState);
  }


  render(){
    if (this.props.init){
      const { editorState, toolbarConfig, toggleAlignment, linkToggle, linkPrompt: { showLinkPrompt, urlValue, linkCaption } } = this.props;
      return (
        <div className={ styles.container }>
            <Toolbar key="toolbar"
              onToggle={ this.changeState }
              toolbarConfig={ toolbarConfig }
              editorState={ editorState }
              linkToggle={ linkToggle }
              toggleAlignment={ toggleAlignment }
            />
            <LinkPrompt key="LinkModal"
              isOpen={ showLinkPrompt }
              editorState={ editorState } urlValue={ urlValue } linkCaption={ linkCaption }
              changeState={ linkToggle }
              onConfirm={ this.changeState }
              captions={ toolbarConfig.linkCaptions }
              onClose={ () => {
                setImmediate(()=>{
                  this.refs.editorArea.editor.focus();
                });
              } }
            />
            <EditorArea
              ref="editorArea"
              key="editorArea"
              editorState={ editorState }
              blockRenderMap={ blockRenderMap }
              blockRendererFn={ blockRendererFn }
              changeState={ this.changeState }
              handleKeyCommand={ this.handleKeyCommand }
              handleReturn={ this.handleReturn }
              onTab={ this.onTab }
            />
        </div>
      );
    }
    return <div className={ styles.container }></div>;
  }
}

export default compose(
  connect(({ editor }) => {
    return editor;
  }, { changeEditorState: editorStateChange, toggleAlignment: alignmentToggle, linkToggle: linkPromptChangeState }),
  pure
)(EditorComponent);