import { fromJS } from 'immutable';
import { BLOCK_TYPE } from 'draft-js-utils';
import { EditorState, SelectionState, ContentBlock, Entity, Modifier, BlockMapBuilder, genKey } from 'draft-js';

export function alignBlock(contentState, selection, alignment) {
  return Modifier.setBlockData(contentState, selection, fromJS({ alignment }));
}

export function getDraftStates(editorState: EditorState){
  let contentState = editorState.getCurrentContent();
  const selectionState = editorState.getSelection();
  const block = contentState.getBlockForKey(selectionState.getStartKey());
  return {
    contentState,
    selectionState,
    block,
  };
}

export function isListItem(block){
  let blockType = block.getType();
  return (
    blockType === BLOCK_TYPE.UNORDERED_LIST_ITEM ||
    blockType === BLOCK_TYPE.ORDERED_LIST_ITEM
  );
}

export function changeBlockType(
  editorState,
  blockKey,
  newType,
) {
  let content = editorState.getCurrentContent();
  let block = content.getBlockForKey(blockKey);
  let type = block.getType();
  if (type === newType) {
    return editorState;
  }
  let newBlock = block.set('type', newType);
  let newContent = content.merge({
    blockMap: content.getBlockMap().set(blockKey, newBlock),
  });
  return EditorState.push(
    editorState,
    newContent,
    'change-block-type'
  );
}

export default function changeBlockDepth(
  editorState,
  blockKey,
  newDepth,
) {
  let content = editorState.getCurrentContent();
  let block = content.getBlockForKey(blockKey);
  let depth = block.getDepth();
  if (depth === newDepth) {
    return editorState;
  }
  let newBlock = block.set('depth', newDepth);
  let newContent = content.merge({
    blockMap: content.getBlockMap().set(blockKey, newBlock),
  });
  return EditorState.push(
    editorState,
    newContent,
    'adjust-depth'
  );
}

export function handleReturnEmptyListItem(editorState, changeState){
  const selection = editorState.getSelection();
  if (selection.isCollapsed()) {
    const blockKey = selection.getStartKey();
    const contentState = editorState.getCurrentContent();
    const block = contentState.getBlockForKey(blockKey);
    if (isListItem(block) && block.getLength() === 0) {
      const depth = block.getDepth();
      const newState = (depth === 0) ?
        changeBlockType(editorState, blockKey, BLOCK_TYPE.UNSTYLED) :
        changeBlockDepth(editorState, blockKey, depth - 1);
      changeState(newState);
      return true;
    }
  }
  return false;
}


export function getActiveEntityAtCursor(editorState: EditorState){
  const { selectionState, block } = getDraftStates(editorState);
  const focusedChar = block.getCharacterList().get(selectionState.getStartOffset() - 1);
  let activeEntityAtCursor = 'NONE';
  if (focusedChar){
    const entityKey = focusedChar.getEntity();
    activeEntityAtCursor = entityKey && Entity.get(entityKey) && Entity.get(entityKey).getType() || activeEntityAtCursor;
  }
  return activeEntityAtCursor;
}

export function removeCustomBlocksOnBackspace(editorState: EditorState, command: string, blockMap: Array<string>){
  const selectionState = editorState.getSelection();
  const contentState = editorState.getCurrentContent();
  const block = contentState.getBlockForKey(selectionState.getStartKey());
  const previousBlockKey = contentState.getKeyBefore(block.getKey());
  const offset = selectionState.getStartOffset();
  if (previousBlockKey && command === 'backspace') {
    const previousBlock = contentState.getBlockForKey(previousBlockKey);
    const targetRange = new SelectionState({
      anchorKey: previousBlockKey,
      anchorOffset: 0,
      focusKey: previousBlockKey,
      focusOffset: 0,
    });
    if (selectionState.isCollapsed() && blockMap.includes(previousBlock.getType()) && offset === 0){
      const newState = EditorState.push(editorState, Modifier.setBlockType(contentState, targetRange, 'unstyled'));
      return EditorState.forceSelection(newState, targetRange);
    }
  }
  return false;
}

export function reAlignBlock(editorState: EditorState, alignment: string, toggleAlignment: Function){
  let newEditorState = editorState;
  let { contentState, selectionState, block } = getDraftStates(editorState);
  const blockData = block.getData();
  const isEmptyBlock = blockData.isEmpty();
  const blockAlignmentValue = blockData.get('alignment');
  if ((isEmptyBlock || !blockAlignmentValue) && selectionState.isCollapsed()) {
    contentState = alignBlock(contentState, selectionState, alignment);
    newEditorState = EditorState.push(editorState, contentState); // eslint-disable-line no-param-reassign
  }
  if (!isEmptyBlock && blockAlignmentValue !== alignment){
    toggleAlignment({ alignment: blockAlignmentValue, });
  }
  return newEditorState;
}

export function addBlock(editorState, selection, type, data, text = ' ') {
  const currentContentState = editorState.getCurrentContent();
  const currentSelectionState = selection;

  // in case text is selected it is removed and then the block is appended
  const afterRemovalContentState = Modifier.removeRange(
    currentContentState,
    currentSelectionState,
    'backward'
  );

  // deciding on the postion to split the text
  const targetSelection = afterRemovalContentState.getSelectionAfter();
  const blockKeyForTarget = targetSelection.get('focusKey');
  const block = currentContentState.getBlockForKey(blockKeyForTarget);
  let insertionTargetSelection;
  let insertionTargetBlock;

  // In case there are no characters or entity or the selection is at the start it
  // is safe to insert the block in the current block.
  // Otherwise a new block is created (the block is always its own block)
  const isEmptyBlock = block.getLength() === 0 && block.getEntityAt(0) === null;
  const selectedFromStart = currentSelectionState.getStartOffset() === 0;
  if (isEmptyBlock || selectedFromStart) {
    insertionTargetSelection = targetSelection;
    insertionTargetBlock = afterRemovalContentState;
  } else {
    // the only way to insert a new seems to be by splitting an existing in to two
    insertionTargetBlock = Modifier.splitBlock(afterRemovalContentState, targetSelection);

    // the position to insert our blocks
    insertionTargetSelection = insertionTargetBlock.getSelectionAfter();
  }

  // TODO not sure why we need it …
  const newContentStateAfterSplit = Modifier.setBlockType(insertionTargetBlock, insertionTargetSelection, type);

  const fragmentArray = [
    new ContentBlock({
      key: genKey(),
      type
    }),

    new ContentBlock({
      key: genKey(),
      type: 'unstyled',
      text: ''
    }),
  ];

  // create fragment containing the two content blocks
  const fragment = BlockMapBuilder.createFromArray(fragmentArray);

  // replace the contentblock we reserved for our insert
  const contentStateWithBlock = Modifier.replaceWithFragment(
    newContentStateAfterSplit,
    insertionTargetSelection,
    fragment
  );

  // update editor state with our new state including the block
  const newState = EditorState.push(
    editorState,
    contentStateWithBlock,
    `insert-${type}`
  );
  return EditorState.forceSelection(newState, contentStateWithBlock.getSelectionAfter());
}

export function insertBlankSpace(editorState){
  const selectionState = editorState.getSelection();
  const endOffset = selectionState.getEndOffset();
  const selection = selectionState.merge({
    focusOffset: endOffset,
    anchorOffset: endOffset,
    hasFocus: true
  });
  const newContent = Modifier.insertText(editorState.getCurrentContent(), selection, ' ');
  return EditorState.push(editorState, newContent);
}

export function getSelectedText(selectionState, editorState){
  const start = selectionState.getStartOffset();
  const end = selectionState.getEndOffset();
  const contentBlock = editorState.getCurrentContent().getBlockForKey(selectionState.getStartKey());
  return contentBlock.getText().slice(start, end);
}
